----------------------------------------------------------------------------------
-- Company: Cal Poly SLO
-- Engineer: Kevin Shibley and Konrad Antoniuk
-- 
-- Description: This is the main module for the RAT CPU. In general, this module
--			port maps the other modules together. Some MUXes were created inside this
--			module instead of creating additional modules. Out_port is assigned the 8
--			least significant bits of MULTI_BUS and port_id is assigned the least 8
--			bits of the instruction register output.
--
-- Revisions: Various ways of implementing MUXes were used in the process of debugging
--			the project. The versions not used have been deleted.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity RAT_CPU is
    Port ( IN_PORT : in  STD_LOGIC_VECTOR (7 downto 0);
           RST : in  STD_LOGIC := '0';
           INT_IN : in  STD_LOGIC;
           CLK : in  STD_LOGIC;
           OUT_PORT : out  STD_LOGIC_VECTOR (7 downto 0);
           PORT_ID : out  STD_LOGIC_VECTOR (7 downto 0);
           IO_OE : out  STD_LOGIC);
end RAT_CPU;

architecture Behavioral of RAT_CPU is

-----------------------
-----  COMPONENTS -----
-----------------------

--------------------------- Interrupt Debouncer ------------------------------------

component Debouncer is
    Port ( CLK : in  STD_LOGIC;
			  RST : in  STD_LOGIC;
           BTN_IN : in  STD_LOGIC;
           BTN_OUT : out  STD_LOGIC);
end component;


------------------------------ PROGRAM COUNTER --------------------------------------
Component ProgramCounter is
    Port ( PC_INC : IN STD_LOGIC;
           PC_OE : in  STD_LOGIC;
           CLK : in  STD_LOGIC;
           RST : in STD_LOGIC;
			  PC_LD: in STD_LOGIC;
           D_IN : in  STD_LOGIC_VECTOR (9 downto 0);
           PC_TRI : out  STD_LOGIC_VECTOR (9 downto 0);
           pc_COUNT : out  STD_LOGIC_VECTOR (9 downto 0));
end Component;

------------------------------ PROG_ROM --------------------------------------

Component prog_rom is
   port (     ADDRESS : in std_logic_vector(9 downto 0); 
          INSTRUCTION : out std_logic_vector(17 downto 0); 
                  CLK : in std_logic);  
end component;

------------------------------ REGISTER FILE --------------------------------------

Component RegisterFile is
    Port ( D_IN   : in     STD_LOGIC_VECTOR (7 downto 0);
           DX_OUT : out  	STD_LOGIC_VECTOR (7 downto 0);
           DY_OUT : out    STD_LOGIC_VECTOR (7 downto 0);
           ADRX   : in     STD_LOGIC_VECTOR (4 downto 0);
           ADRY   : in     STD_LOGIC_VECTOR (4 downto 0);
			  DX_OE  : in     STD_LOGIC;
           WE     : in     STD_LOGIC;
           CLK    : in     STD_LOGIC);
end component;


------------------------------ SCRATCH PAD --------------------------------------

Component ScratchPad is
    Port ( SP_ADRS	: in     STD_LOGIC_VECTOR (7 downto 0);
           SP_WE		: in     STD_LOGIC;
           SP_OE		: in     STD_LOGIC;
           SP_DATA	: inout  STD_LOGIC_VECTOR (9 downto 0);
           CLK    	: in     STD_LOGIC);
end component;

------------------------------ CONTROL --------------------------------------

component CONTROL is
    Port ( CLK           : in   STD_LOGIC;
           C             : in   STD_LOGIC;
           Z             : in   STD_LOGIC;
           INT           : in   STD_LOGIC;
			  RST           : in   STD_LOGIC;
			  OPCODE_HI_5   : in   STD_LOGIC_VECTOR (4 downto 0);
			  OPCODE_LO_2   : in   STD_LOGIC_VECTOR (1 downto 0);
			  
           PC_LD         : out  STD_LOGIC;
           PC_INC        : out  STD_LOGIC;
           PC_SEL   		 : out  STD_LOGIC_VECTOR (1 downto 0);
			  PC_RESET      : out  STD_LOGIC;
           PC_OE         : out  STD_LOGIC;
           SP_LD         : out  STD_LOGIC;
           SP_MUX_SEL    : out  STD_LOGIC_VECTOR (1 downto 0);
			  SP_RESET      : out  STD_LOGIC;
           RF_WR         : out  STD_LOGIC;
			  RF_WR_SEL     : out  STD_LOGIC_VECTOR (1 downto 0);
           RF_OE         : out  STD_LOGIC;
           REG_IMMED_SEL : out  STD_LOGIC;
           ALU_SEL       : out  STD_LOGIC_VECTOR (3 downto 0);
           SCR_WR        : out  STD_LOGIC;
           SCR_OE        : out  STD_LOGIC;
           SCR_ADDR_SEL  : out  STD_LOGIC_VECTOR (1 downto 0);
           FLAG_SEL      : out  STD_LOGIC;
           SHAD_LD       : out  STD_LOGIC;
           C_FLAG_LD     : out  STD_LOGIC;
           C_FLAG_SET    : out  STD_LOGIC;
           C_FLAG_CLR    : out  STD_LOGIC;
           Z_FLAG_LD     : out  STD_LOGIC;
           Z_FLAG_SET    : out  STD_LOGIC;
           Z_FLAG_CLR    : out  STD_LOGIC;
           I_FLAG_SET    : out  STD_LOGIC;
           I_FLAG_CLR    : out  STD_LOGIC;
			  IO_OE			 : out  STD_LOGIC);
end component;

------------------------------ FLAG REGISTER --------------------------------------

component FLAG_REG is
    Port ( IN_FLAG : in  STD_LOGIC;
           LD : in  STD_LOGIC;			  			  
           SET : in  STD_LOGIC;
           CLR : in  STD_LOGIC;
           CLK : in  STD_LOGIC;
           OUT_FLAG : out  STD_LOGIC := '0');
end component;

------------------------- ARITHMETIC LOGIC UNIT --------------------------------------

component ALU is
    Port ( A : in  STD_LOGIC_VECTOR (7 downto 0);
           B : in  STD_LOGIC_VECTOR (7 downto 0);
           C_IN : in  STD_LOGIC;
           SEL : in  STD_LOGIC_VECTOR (3 downto 0);
           SUM : out  STD_LOGIC_VECTOR (7 downto 0);
           C_FLAG : out  STD_LOGIC;
           Z_FLAG : out  STD_LOGIC);
end component;



------------------------- STACK POINTER ------------------------------------------------

component StackPointer is
    Port ( D_IN : in  STD_LOGIC_VECTOR (7 downto 0);
           SP_LD : in  STD_LOGIC;
           RST : in  STD_LOGIC;
           CLK : in  STD_LOGIC;
           STK_PNTR : out  STD_LOGIC_VECTOR (7 downto 0):= "00000000");
end component;



-----------------------
------- SIGNALS -------
-----------------------


---- CONTROL UNIT signals -----

-- signals such as CLK, INT, and RST come from the inputs of this module and are excluded here
signal	PC_LD, PC_INC, PC_RESET, PC_OE, SP_LD, SP_RESET, SCR_WR, SCR_OE : STD_LOGIC;
signal	RF_WR, RF_OE, REG_IMMED_SEL : STD_LOGIC; --REG FILE SIGNALS
signal	C, Z, C_FLAG_LD, C_FLAG_SET, C_FLAG_CLR, Z_FLAG_LD, Z_FLAG_SET, Z_FLAG_CLR : STD_LOGIC; -- FLAG SIGNALS
signal	SHAD_LD, FLAG_SEL, I_FLAG_SET, I_FLAG_CLR, shad_z_out, shad_c_out, C_mux_out, Z_mux_out: std_logic; -- FLAG SIGNALS
signal	PC_SEL, PC_MUX_SEL, SP_MUX_SEL, RF_WR_SEL, SCR_ADDR_SEL : STD_LOGIC_VECTOR (1 downto 0);
signal	ALU_SEL : STD_LOGIC_VECTOR (3 downto 0);
signal	I, INT_FLAG : STD_LOGIC := '1';


-------- intermediate signals ---------
signal PC, PC_MUX_OUT : STD_LOGIC_VECTOR (9 downto 0); -- from PC_COUNT to ADDRESS of prog_rom
signal IR : STD_LOGIC_VECTOR (17 downto 0); -- output from prog_rom, goes to RegisterFile, ALU, and PORT_ID
signal ALU_DATA : STD_LOGIC_VECTOR (7 downto 0); -- output SUM of ALU, goes to RF_WR_SEL mux
signal REG_Y, B_IN: STD_LOGIC_VECTOR (7 downto 0); -- output from RegisterFile, goes to ALU (Y becomes B_IN through REG_IMMED_SEL)
signal REG_IN : STD_LOGIC_VECTOR (7 downto 0); -- signal between the register file's mux and its input
signal C_temp, Z_temp : STD_LOGIC; -- output of ALU, goes to input of FLAG_REG
signal MULTI_BUS : STD_LOGIC_VECTOR (9 downto 0);
signal SCR_ADDR : STD_LOGIC_VECTOR (7 downto 0); -- signal controled by SCR_ADDR_SEL
signal STK_PNTR, SP_MUX_OUT : STD_LOGIC_VECTOR (7 downto 0);
signal INTERRUPT : STD_LOGIC;


begin

out_port <= MULTI_BUS(7 downto 0);
port_id <= IR(7 downto 0);


---------- RegisterFile MUX ---------------------------


	with RF_WR_SEL	select
	REG_IN <=	in_port when "00",
					MULTI_BUS(7 downto 0) when "01",
					ALU_DATA when "10",
					"00000000" when others;
					

				
----------- ALU MUX ----------------------------------	


	B_IN <= IR(7 downto 0) when REG_IMMED_SEL = '1' else
			REG_Y;



----------- SCRATCH PAD MUX ----------------------------

		
	with SCR_ADDR_SEL	select
	SCR_ADDR <=	IR(7 downto 0) when "01",
					STK_PNTR when "10",
					STK_PNTR - "1" when "11",
					REG_Y when others;
									
									

------------  PORT MAPPING -----------------------------


--INT_DEB: Debouncer
--    port map ( CLK => CLK,
--			  RST => RST,
--           BTN_IN => INT_IN,
--           BTN_OUT => INTERRUPT);


ArithmeticUnit: ALU PORT MAP ( A => MULTI_BUS(7 downto 0),
           B => B_IN,
           C_IN  => C,
           SEL => ALU_SEL,
           SUM  => ALU_DATA,
           C_FLAG  => C_temp,
           Z_FLAG => Z_temp);
			  

C_FLAG :  FLAG_REG port map (
           IN_FLAG => C_mux_out,
           LD => C_FLAG_LD	,	  
           SET => C_FLAG_SET,
           CLR => C_FLAG_CLR,
           CLK => CLK,
           OUT_FLAG => C);
           
C_FLAG_SHAD :  FLAG_REG port map (
              IN_FLAG => C,
              LD => SHAD_LD,		  
              SET => '0',
              CLR => '0',
              CLK => CLK,
              OUT_FLAG => shad_c_out);
              
with FLAG_SEL select
   C_mux_out <= C_temp when '0',
                shad_c_out when others;
                  

Z_FLAG :  FLAG_REG port map (
           IN_FLAG => Z_mux_out,
           LD => Z_FLAG_LD,		  
           SET => Z_FLAG_SET,
           CLR => Z_FLAG_CLR,
           CLK => CLK,
           OUT_FLAG => Z);
           
Z_FLAG_SHAD :  FLAG_REG port map (
              IN_FLAG => Z,
              LD => SHAD_LD,		  
              SET => '0',
              CLR => '0',
              CLK => CLK,
              OUT_FLAG => shad_z_out);
              
with FLAG_SEL select
   Z_mux_out <= Z_temp when '0',
                shad_z_out when others;
                  


I_FLAG :  FLAG_REG port map (
           IN_FLAG => '0',
           LD => '0',		  
           SET => I_FLAG_SET,
           CLR => I_FLAG_CLR,
           CLK => CLK,
           OUT_FLAG => INT_FLAG);


I <= (INT_FLAG AND INT_IN);	-- If you want the IFLAG debounce, make INT_IN INTERRUPT

CONTROL_UNIT: CONTROL PORT MAP ( CLK => CLK,
           C => C,
           Z => Z,
           INT => I,
			  RST => RST,
			  OPCODE_HI_5 => IR(17 downto 13),
			  OPCODE_LO_2 => IR(1 downto 0),
           PC_LD => PC_LD,
           PC_INC => PC_INC,
           PC_SEL => PC_SEL,
			  PC_RESET => PC_RESET,
           PC_OE => PC_OE,
           SP_LD => SP_LD,
           SP_MUX_SEL => SP_MUX_SEL,
			  SP_RESET => SP_RESET,
           RF_WR => RF_WR,
			  RF_WR_SEL => RF_WR_SEL,
           RF_OE => RF_OE,
           REG_IMMED_SEL => REG_IMMED_SEL,
           ALU_SEL => ALU_SEL,
           SCR_WR => SCR_WR,
           SCR_OE => SCR_OE,
           SCR_ADDR_SEL => SCR_ADDR_SEL,
           FLAG_SEL => FLAG_SEL,
			  SHAD_LD => SHAD_LD,
           C_FLAG_LD => C_FLAG_LD,
           C_FLAG_SET => C_FLAG_SET,
           C_FLAG_CLR => C_FLAG_CLR,
           Z_FLAG_LD => Z_FLAG_LD,
           Z_FLAG_SET => Z_FLAG_SET,
           Z_FLAG_CLR => Z_FLAG_CLR,
           I_FLAG_SET => I_FLAG_SET,
           I_FLAG_CLR => I_FLAG_CLR,
--			  WRITE_STROBE => ,
			  IO_OE => IO_OE);



SP: StackPointer PORT MAP ( 
           D_IN => SP_MUX_OUT,
           SP_LD => SP_LD,
           RST => SP_RESET,
           CLK => CLK,
           STK_PNTR => STK_PNTR);
           
with SP_MUX_SEL select
   SP_MUX_OUT <= MULTI_BUS(7 downto 0) when "00",
                 STK_PNTR - 1 when "10",
                 STK_PNTR + 1 when "11",
                 "00000000" when others;

			  
			  
SCR: ScratchPad PORT MAP ( SP_ADRS => SCR_ADDR,	
				SP_WE => SCR_WR,
            SP_OE => SCR_OE,
            SP_DATA => MULTI_BUS,
				CLK  => CLK);
				


Reg_File:	RegisterFile PORT MAP ( D_IN => REG_IN,
           DX_OUT => MULTI_BUS(7 downto 0),
           DY_OUT => REG_Y,
           ADRX => IR(12 downto 8),
           ADRY => IR(7 downto 3),
			  DX_OE => RF_OE,
           WE => RF_WR,
           CLK => CLK);


P_ROM: prog_rom PORT MAP ( ADDRESS => PC,
          INSTRUCTION => IR,
          CLK => CLK);  


COUNTER: ProgramCounter PORT MAP
         ( PC_INC => pc_inc,
           PC_OE => PC_OE,
           CLK => CLK,
           RST => PC_RESET,
			  PC_LD => PC_LD,
           D_IN => pc_mux_out,
           PC_TRI => MULTI_BUS,
           pc_COUNT => PC);

with PC_SEL select
   pc_mux_out <= IR(12 downto 3) when "00",
                 MULTI_BUS when "01",
                 "1111111111" when others;                 



end Behavioral;

